!  DLLGeeStrrain5.f90 
    
subroutine DLLGeeStrrain5 (ncomp,exppar,simparA,simparG,table,simulation)

  ! Expose subroutine DLLGeeStrrain5 to users of this DLL
  !
  !DEC$ ATTRIBUTES DLLEXPORT::DLLGeeStrrain5
  !DEC$ ATTRIBUTES STDCALL::DLLGeeStrrain5


!	Scans the unit sphere.
!	g-strain as in
!	J.Magn.Reson. 61 (1985) p.229, equation [34]; and
!	Advanced EPR (Hoff ed. 1989) p.800, equation [19].
!
!   Extention to double g-value and field range to avoid cut-off artifacts
!
!	WR Hagen  Thise version V2.0 checked on [20210205]


implicit none

! input variables
	integer*4	ncomp, itheta
    real*8		exppar(3), simparA(3), simparG(10,5)
    real*8		table(40000), simulation(1024)

! primary variables
    integer*4   npoint, n200, ngpoint, iret
    real*8      gxx, gyy, gzz, deltaxx, deltayy, deltazz, deltaxy, deltaxz, deltayz, ratio
    real*8      resid, bstep, gmin, gmax, gstep
    real*8      gspec(98304), bspec(1026)
    real*8		frequency, bmin, bmax, bminreal

! internal variables    
    integer*4   i, ntheta, nphi, iphi, jtheta
    real*8      pi, deltheta, delphi, phi
    real*8      lz,lz2,ly,ly2,lx,lx2
    real*8      gx2,gy2,gz2,gx4,gy4,gz4,gdelxy,gdelxz,gdelyz,resid2,gsum2
    real*8      sintheta,sintheta2,sinphi,sinphi2,cosphi,cosphi2
    real*8      gten2,gten,deltag,xnint,prob
    integer*4   nleft

        frequency	= exppar(1)
        bmin		= exppar(2)
        bmax		= exppar(3)
        
	    pi = 3.1415926
		npoint	= 1024
		ntheta	= simparA(1)
		nphi	= simparA(2)
        resid	= simparA(3)
	
		gxx		= simparG(1,ncomp)
		gyy		= simparG(2,ncomp)
		gzz		= simparG(3,ncomp)
		deltaxx	= simparG(4,ncomp)
		deltayy	= simparG(5,ncomp)
		deltazz	= simparG(6,ncomp)
		deltaxy	= simparG(7,ncomp)
		deltaxz	= simparG(8,ncomp)
		deltayz	= simparG(9,ncomp)
		ratio	= simparG(10,ncomp)
        
        DO  i=1,npoint
			simulation(i) = 0.
        END DO
        
        bminreal= bmin	

		! Adjust number of points for high g-values (low fields):
		ngpoint = 1024*2                        !twice range
		IF (bmin .LT. 1600.) ngpoint = 2048*2   !"
		IF (bmin .LT. 800.)  ngpoint = 4096*2   !"
		IF (bmin .LT. 400.)  ngpoint = 8192*2   !"
		IF (bmin .LT. 200.)  ngpoint = 16384*2  !"

        IF (bmin .LT. 200.) bmin = 200.
        
		DO	i = 1, 3*ngpoint
			gspec(i) = 0.
        END DO
        
        ! Avoid fields of less than 200 gauss:
		n200 = ((bmax-bmin)/(bmax-bminreal)) * REAL(npoint) 
		bstep = (bmax-bmin)/(n200-1.)

	    gmin  = 0.714484 * frequency / (bmax*2.)    !twice range
		gmax  = 0.714484 * frequency / bmin
		gstep = (gmax-gmin) / (ngpoint-1.)
        
		IF (resid .LT. gstep) resid = gstep

		! Next lines define starting points for midpoint integration for a uniformly spaced mesh 
        ! that accounts for periodicity. Ref: J.Magn.Reson. 61 (1985) p.235, equation [8].
		deltheta=2./ntheta
		lz=1.+deltheta/4.
		nphi=nphi+MOD(nphi,2)-1
		delphi=pi*2./nphi
		phi=-delphi/8.
	
		resid2 = resid * resid
		gx2=gxx*gxx
		gy2=gyy*gyy
		gz2=gzz*gzz
		gdelxy = SQRT(gxx*gyy) * deltaxy
		gdelxz = SQRT(gxx*gzz) * deltaxz
		gdelyz = SQRT(gyy*gzz) * deltayz
		gsum2=gx2+gy2+gz2
		gx4=gx2*gx2
		gy4=gy2*gy2
		gz4=gz2*gz2

 	DO	itheta=1,ntheta
		lz=lz-deltheta
		lz2=lz*lz
		sintheta2=1-lz2
		sintheta=SQRT(sintheta2)
	
!------------------- phi loop --------------------
	  DO iphi=1,nphi
	    phi=phi+delphi
	    sinphi=SIN(phi)
	    sinphi2=sinphi*sinphi
	    cosphi=COS(phi)
	    cosphi2=cosphi*cosphi
	    lx2=cosphi2*sintheta2
	    ly2=sinphi2*sintheta2
	    lx=cosphi*sintheta
	    ly=sinphi*sintheta
	    gten2=gx2*lx2+gy2*ly2+gz2*lz2
	    gten=SQRT(gten2)
		!----------- line width calculation -----
	    deltag = lx2*gxx*deltaxx + ly2*gyy*deltayy + lz2*gzz*deltazz &
     	    + 2. * (lx*ly*gdelxy + lx*lz*gdelxz + ly*lz*gdelyz)

		!----------- residual broadening --------
	    deltag = deltag * deltag / gten2 + resid2
	    deltag = SQRT(deltag)

		!----------- transition probability -----
	    prob=gsum2-(gx4*lx2+gy4*ly2+gz4*lz2)/gten2

		!----------- compute line shape ---------
	    CALL shape !(gten,deltag,prob)
        
      END DO	! of phi loop

	END DO		! of cos(theta) loop
      
		!-- normalize and frame backshift -------
		!  (frameshift is in Subroutine shape)
		xnint=ntheta*nphi
		DO i=1,ngpoint
			gspec(i)= ratio * gspec(i+ngpoint)/gstep/xnint
		END DO

		CALL gtob  ! from g-space to B-space

		IF (bminreal .LT. 200.) THEN
		! Shift the n200-point spectrum to the right, and
		! pad the first LE.200 gauss with zero intensities
			nleft = npoint - n200
			DO i = 1, n200
				simulation(i+nleft) = bspec(i)
			END DO
			DO i = 1, nleft
				simulation(i) = 0.
			END DO
		ELSE    
			DO  i = 1, npoint
				simulation(i) = bspec(i)
			END DO
        END IF

	RETURN

CONTAINS
!==============================================================================
SUBROUTINE shape !(gten,deltag,prob)

! This subroutine puts a gaussian line over the resonance position in g-space
	
    real*8		range,factor,frac,trans
    integer*4	ngauss,left,ial,ifac,iar,ileft,ilecut,iright,iricut,iend

	range=deltag*3.157/gstep
	factor=12800./range

    !------- discard lines broader than whole scan --------
	IF (range.GT.REAL(ngpoint)) RETURN
	trans=prob*factor
	ngauss=range-.5
	left=(gten-gmin)/gstep
	frac=(gten-gmin-left*gstep)/gstep
	ial=1+frac*factor
	IF (ial .LT. 1) RETURN               
	ifac=factor+.5
	iar=-ial+2+ifac
	!	frame shift to discard lines totally outside the field window
	!	avoids repeated IF statements in the next unrolled DO loop
	ileft=left+ngpoint+1
	ilecut=left-ngauss-(ngpoint-1)
	iright=ileft+1
	iricut=iright+ngauss-ngpoint
	IF (ilecut.GE.0.OR.iricut.LE.0) RETURN
	IF (ilecut.LE.-ngpoint) RETURN
	!----- start of the unrolled DO loop
	gspec(ileft)=gspec(ileft)+trans*table(ial)
	ial=ial+ifac
	gspec(ileft-1)=gspec(ileft-1)+trans*table(ial)
	ial=ial+ifac
	gspec(ileft-2)=gspec(ileft-2)+trans*table(ial)
	ial=ial+ifac
	gspec(ileft-3)=gspec(ileft-3)+trans*table(ial)
	ial=ial+ifac
    gspec(iright)=gspec(iright)+trans*table(iar)
    iar=iar+ifac
    gspec(iright+1)=gspec(iright+1)+trans*table(iar)
    iar=iar+ifac
    gspec(iright+2)=gspec(iright+2)+trans*table(iar)
    iar=iar+ifac
    gspec(iright+3)=gspec(iright+3)+trans*table(iar)
    iar=iar+ifac
    	
    IF (ngauss.LT.5) RETURN
    iend=(ngauss)/4
    	
    DO i=1,iend
		ileft=ileft-4
		gspec(ileft)=gspec(ileft)+trans*table(ial)
		ial=ial+ifac
        gspec(ileft-1)=gspec(ileft-1)+trans*table(ial)
		ial=ial+ifac
    	gspec(ileft-2)=gspec(ileft-2)+trans*table(ial)
    	ial=ial+ifac
    	gspec(ileft-3)=gspec(ileft-3)+trans*table(ial)
    	ial=ial+ifac
    	iright=iright+4
    	gspec(iright)=gspec(iright)+trans*table(iar)
    	iar=iar+ifac
    	gspec(iright+1)=gspec(iright+1)+trans*table(iar)
    	iar=iar+ifac
    	gspec(iright+2)=gspec(iright+2)+trans*table(iar)
    	iar=iar+ifac
    	gspec(iright+3)=gspec(iright+3)+trans*table(iar)
    	iar=iar+ifac
    END DO

	RETURN
END SUBROUTINE shape 
!==============================================================================
SUBROUTINE gtob
! This subroutine converts spectrum in g-space to field space spectrum with equally spaced x steps 

    integer*4	i,n1
    real*8		g1,frac,bval,gval

	DO i=1,npoint+2
		bspec(i)=0.
	END DO

	DO i=1,n200
		bval=bmin+(i-1)*bstep
		gval=0.714484*frequency/bval
		n1=(gval-gmin)/gstep
		g1=gmin+n1*gstep
		frac=(gval-g1)/gstep
		bspec(i)=(1.-frac)*gspec(n1+1)+frac*gspec(n1+2)
		bspec(i)=bspec(i)*gval	! Aasa (1/g) and volume element (g**2) correction
	END DO

	RETURN
END SUBROUTINE gtob
!==============================================================================

end subroutine DLLGeeStrrain5
