!  DLL GaussianLine.f90 
!
!  This DLL computes a Gaussian at bres with standard deviation gamma
!  by searching the huge integer table DLL_GaussianTable
!  WRHagen 1982, 20060908, 20191224
!
subroutine DLLGaussianLine(npoints,table,bres,bmin,bdist,gamma,prob,gaussian)

!  NOTE: gamma is HWHH, that is sigma * 1.177

IMPLICIT NONE
  !DEC$ ATTRIBUTES DLLEXPORT::DLLGaussianLine
  !DEC$ ATTRIBUTES STDCALL::DLLGaussianLine

  ! Variables
    INTEGER     npoints
    REAL        table(40000)
    REAL        bres, bmin, bdist, gamma, prob
    REAL        gaussian(npoints)
    
    REAL        range, rpoint, point, factor, trans, fij
    INTEGER     mpoint, intpoint, jfi
    INTEGER     k

 ! Body of DLLGaussianLine

    DO  k=1,npoints
        gaussian(k) = 0.
    END DO
    IF (prob .LE. 0.0) RETURN
    range = - gamma * 3.65
    rpoint = (bres + range - bmin) / bdist
    mpoint = rpoint + 0.5
    point = rpoint - 2. * range / bdist - 0.5
    intpoint = point
    factor = -15000. * bdist / range
    trans = prob * factor
    IF (mpoint .LT. 1) mpoint = 1
    IF (intpoint .GT. npoints) intpoint = npoints
    fij = 1. + (mpoint - rpoint) * factor
    DO  k=mpoint, intpoint
        jfi = fij + 0.5
        gaussian(k) = gaussian(k) + trans * table(jfi+5000)
        fij = fij + factor
    END DO

end subroutine DLLGaussianLine
