!  DLLSTRAINWIDTH.f90 
!
!  The g-strain linewidth is calculated for a given orientation
!  and is converted to a field width at given frequency and resonance field
!  WRHagen 20191211, 20191222
!
subroutine DLLSTRAINWIDTH (frequency, bfield, lvector, gvector, dvector, bwidth)
  !
  !DEC$ ATTRIBUTES DLLEXPORT::DLLSTRAINWIDTH
  !DEC$ ATTRIBUTES STDCALL::DLLSTRAINWIDTH

    implicit none

  ! Variables
    REAL*8      frequency, bfield
    REAL*8      lvector(3), gvector(3), dvector(7)
    REAL*8      bwidth(2)
    
    REAL*8      lx, ly, lz, gx, gy, gz, dx, dy, dz, dxy, dxz, dyz, res
    REAL*8      gten, delta, g, bplus, bmin, bcorrected

 ! Body of DLLSTRAINWIDTH
    
    lx = lvector(1)
    ly = lvector(2)
    lz = lvector(3)
    
    gx = gvector(1)
    gy = gvector(2)
    gz = gvector(3)
    
    dx = dvector(1)
    dy = dvector(2)
    dz = dvector(3)
    dxy= dvector(4)
    dxz= dvector(5)
    dyz= dvector(6)
    res= dvector(7)
    
    gten = gx*gx*lx*lx + gy*gy*ly*ly + gz*gz*lz*lz
    gten = SQRT(gten)
    
    delta = lx*ly*SQRT(gx*gy)*dxy + lx*lz*SQRT(gx*gz)*dxz + ly*lz*SQRT(gy*gz)*dyz
    delta = 2.*delta + lx*lx*gx*dx + ly*ly*gy*dy + lz*lz*gz*dz
    delta = delta / gten
    
    delta = delta*delta + res*res
    delta = SQRT(delta)
    
    bcorrected = bfield
    IF (bcorrected .EQ. 0.0) bcorrected = 0.01
    g = 0.714484 * frequency / bcorrected
    bplus = 0.714484 * frequency / (g-(delta/2.))
    bmin  = 0.714484 * frequency / (g+(delta/2.))
    bwidth(1) = bplus - bmin
    bwidth(2) = bwidth(1)

end subroutine DLLSTRAINWIDTH
