! Subroutine GaussianTable.F90
!
! PURPOSE:  Avoiding many floating point operations in EPR simulators
!           by creating a huge table for a digital Gaussian line shape
!           so that Gaussians in EPR spectra can be constructed from this table
!           by linear interpolation
!
! AUTHOR:   W.R. Hagen (Ann Arbor, 1982) [J Magn Reson 1985, 61: 220]
! UPDATE:   8 sept 2006
!
subroutine GaussianTable (table)


  !
  IMPLICIT NONE
  !DEC$ ATTRIBUTES DLLEXPORT::GaussianTable
  !DEC$ ATTRIBUTES STDCALL::GaussianTable

  ! Declare calling parameters
    REAL*4   table(40000)
  
  ! Declare local variables
    real    elen2
    real    cut    
    real    dela            !steps to one standard deviation
    real    shift           !distance from center in required peak
    real    fexpon          !argument of exponential
    integer ileft           !counter from left
    integer jright          !counter from right  

  ! Body of GaussianTable
    
    elen2   = -alog(2.)
    cut     = 3.65
    dela    = 15000./cut
    shift   = -15000.
    
    DO  ileft = 1, 15000
        jright = 30001 - ileft
        fexpon  = elen2 * (shift/dela)**2
        table(ileft+5000) = exp (fexpon)
        table(jright+5000) = table(ileft+5000)
        shift = shift + 1.
    END DO
    
    do  ileft = 1, 5000
        table(ileft) = 0.
        table(ileft+35000) = 0.
    end do
        
    return

end subroutine GaussianTable
